// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _GRAMGEN_H_
#define _GRAMGEN_H_

#include <vector>
#include <string>
#include <limits>

class GramGen {
public:
	// Grams containing "N" are invalid.
	static const unsigned INVALID_GRAM;

	unsigned gramLen;
	void initialize(unsigned gramLen) {
		this->gramLen = gramLen;
		this->mask = (std::numeric_limits<unsigned>::max() >> (32 - (gramLen * 2)));
	}

	// Methods for generating grams. Grams with "N" character are ignored.
	// Used for index construction.
	bool firstGram(const std::string& str, unsigned startPos, unsigned endPos);
	bool nextGram(const std::string& str);
	bool generateGrams(const std::string& str, std::vector<unsigned>& grams);

	// Methods for generating grams. Grams with "N" character get a special hash.
	// Used for generating grams for reads. We need to know the positions of grams with "N".
	bool readGenerateGrams(const std::string& str, std::vector<unsigned>& grams);
	bool readFirstGram(const std::string& str, unsigned startPos, unsigned endPos,
			std::vector<unsigned>& grams);
	bool readNextGram(const std::string& str, std::vector<unsigned>& grams);

	unsigned getGramHash() {
		return hash;
	}

	unsigned getGramPos() {
		return start;
	}

private:
	bool nextValidGram(const std::string& str, unsigned& start, unsigned& end, unsigned& hash);
	bool readNextValidGram(const std::string& str, unsigned& start, unsigned& end, unsigned& hash,
			std::vector<unsigned>& grams);
	unsigned mask;
	unsigned start;
	unsigned end;
	unsigned hash;
	unsigned endPos;
};

#endif
