// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _HOBBESRUNNER_H_
#define _HOBBESRUNNER_H_

#include "hobbesconf.h"
#include "hobbesstats.h"
#include <fstream>

class Indexer;
class Searcher;
class InputReadQueue;
class ResultStringWriter;
class OutputResultSink;

class HobbesRunner {
public:
	HobbesConf conf;
	HobbesStats stats;

	HobbesRunner() :
		zipFirstIn(NULL), zipSecondIn(NULL) {
	}

	void run();

private:
	std::ifstream* zipFirstIn;
	std::ifstream* zipSecondIn;

	void createInputStreams(std::istream** firstIn, std::istream** secondIn);
	InputReadQueue* createInputReadQueue(std::istream* firstIn, std::istream* secondIn);
	void cleanupInputStreams(std::istream* firstIn, std::istream* secondIn);

	ResultStringWriter* createResultStringWriter();

	void createOutputStream(std::ostream** mappedOut);
	OutputResultSink* createOutputResultSink(std::ostream* mappedOut);
	void cleanupOutputStream(std::ostream* mappedOut);

	Searcher* createSearcher(Indexer* indexer);

	void writeStats(unsigned numMappedReads, unsigned singleMappedReads, unsigned numReadsProcessed,
			unsigned numTotalMappings, unsigned numTotalSingleMappings);
	void writeStatsSummary(std::ostream& out, unsigned long* totalindel, unsigned long* totalmismatch);
};

#endif
