// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _PAIREDENDMERGER_H_
#define _PAIREDENDMERGER_H_

#include "include/search/merger.h"
#include "include/search/singleendmerger.h"
#include "include/search/pairedendverifier.h"

#include <vector>

class PairedEndMerger: public Merger {
public:
	SingleEndMerger* singleEndMerger;

	PairedEndMerger(Indexer* indexer, unsigned distThresh, SingleEndMerger* singleEndMerger) :
			Merger(indexer, distThresh), singleEndMerger(singleEndMerger) {
	}

	void findCandidatesAll(PairedEndRead &read, std::vector<unsigned>& candidates1,
			std::vector<unsigned>& candidates2);

	virtual ~PairedEndMerger() {
		delete singleEndMerger;
	}
};

class PairedEndIndelMerger: public PairedEndMerger {
public:
	PairedEndIndelMerger(Indexer* indexer, unsigned distThresh) :
			PairedEndMerger(indexer, distThresh, new SingleEndIndelMerger(indexer, distThresh)) {
	}
};

class PairedEndHeuristicIndelMerger: public PairedEndMerger {
public:
	PairedEndHeuristicIndelMerger(Indexer* indexer, unsigned distThresh) :
				PairedEndMerger(indexer, distThresh,
						new SingleEndHeuristicIndelMerger(indexer, distThresh)) {
	}
};

class PairedEndHammingMerger: public PairedEndMerger {
public:
	PairedEndHammingMerger(Indexer* indexer, unsigned distThresh) :
			PairedEndMerger(indexer, distThresh,
			// TODO: ALEX. Is this a bug? Should it not be SingleEndHammingMerger?
					new SingleEndIndelMerger(indexer, distThresh)) {
	}
};

#endif
