// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _READ_H_
#define _READ_H_

#include "input/readinfo.h"
#include "index/indexer.h"

#include <string>

class Read {
public:
	std::string* readStr;
	unsigned prefixLen;
	std::vector<unsigned> grams;
	std::vector<unsigned> prefixGrams;
	std::vector<BitVectorInvertedListWithOffset> bitvectorInvertedLists;

	Read() :
		readStr(NULL), prefixLen(0) {
	}

	void setReadString(std::string* readStr) {
		this->readStr = readStr;
	}

	// Set inverted lists for this read.
	void getInvertedLists(Indexer* indexer, GramGen& gramGen);

	// Set Hobbes' optimal prefix calculated via dynamic programming. For production.
	void getPrefixDynamicProgramming(unsigned distThresh, unsigned seedLen, unsigned gramLen);

	// Set prefix using Ed Join method. For testing.
	void getEdJoinPrefix(unsigned distThresh, unsigned tOccurThresh, unsigned gramLen);

	// Set Hobbes' optimal prefix calculated via exhaustive recursion. For testing.
	void getPrefixRecursive(unsigned distThresh, unsigned tOccurThresh, unsigned gramLen);

private:
	// Special empty inverted list for grams containing "N".
	static std::vector<BitVecPair> bitvectorNullList;

	void getInvertedLists(BitVectorInvertedIndex* invIndex);
	unsigned getMinEditErrors(std::vector<unsigned> offsets, unsigned gramLen);
	unsigned getPrefixOffsets(std::vector<unsigned>& offsets, unsigned gramLen,
			std::vector<unsigned>& prefixOffsets);
	void getPrefixRecursive(int lastPos, unsigned distRemain, unsigned partialSum,
			unsigned &minSum, std::vector<unsigned>& minPath, std::vector<unsigned> path, unsigned gramLen);
};

class PairedEndRead: public Read {
public:
	Read first;
	Read second;
	void setReadStringFromReadInfo(PairedEndReadInfo& pairedReadInfo);
	void setReverseReadStringFromReadInfo(PairedEndReadInfo& pairedReadInfo);
};

#endif
