// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _SEARCHER_H_
#define _SEARCHER_H_

#ifndef CANDIS_PREALLOC_SIZE
#define CANDIS_PREALLOC_SIZE 1000
#endif

#ifndef RESULTS_PREALLOC_SIZE
#define RESULTS_PREALLOC_SIZE 1000
#endif

#include "include/hobbesconf.h"
#include "include/hobbesstats.h"
#include "include/gramgen.h"
#include "include/search/read.h"

#include <istream>
#include <string>

class Indexer;

class Searcher {
public:
	Indexer* indexer;
	unsigned long* indels;
	unsigned long* mismatches;

	Searcher(Indexer* indexer, unsigned distThresh, HobbesConf& conf, HobbesStats& stats) :
		indexer(indexer), indels(new unsigned long[2 * distThresh + 1]()),
		mismatches(new unsigned long[2 * distThresh + 1]()), conf(conf), stats(stats),
		distThresh(distThresh), numMappedReads(0), numSingleMappedReads(0),
		totalMappings(0), totalSingleMappings(0) {
		gramGen.initialize(indexer->gramGen.gramLen);
	}

	void findMappings(ReadInfo& readInfo);

	unsigned long getTotalMappings() {
		return totalMappings;
	}

	unsigned long getTotalSingleMappings() {
		return totalSingleMappings;
	}

	unsigned getNumMappedReads() {
		return numMappedReads;
	}

	unsigned getNumSingleMappedReads() {
		return numSingleMappedReads;
	}

	virtual MappingResultType getMappingResultType() = 0;

	virtual ~Searcher() {
		delete[] indels;
		delete[] mismatches;
	}

protected:
	HobbesConf& conf;
	HobbesStats& stats;
	GramGen gramGen;
	unsigned distThresh;
	unsigned numMappedReads;
	unsigned numSingleMappedReads;
	unsigned long totalMappings;
	unsigned long totalSingleMappings;

	virtual void prepare(unsigned readLen) {
	}

	virtual void findAllMappings(ReadInfo& readInfo) = 0;
	virtual void findKMappings(ReadInfo& readInfo) = 0;

	virtual void getStats(std::pair<unsigned, unsigned>& counts) = 0;
	virtual void getBestStats() = 0;

	virtual bool fillRead(Read& read, unsigned& tOccurThresh);

	void getReverseComplementRead(std::string& readStr);
	void generateReverses(ReadInfo& readInfo);
	void getReverseReadQuality(std::string& readQuality);
	void complementChar(char& c);
};

#endif
