// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#ifndef _SINGLEENDMERGER_H_
#define _SINGLEENDMERGER_H_

#include "hobbesconf.h"
#include "search/read.h"
#include "search/merger.h"
#include "search/singleendverifier.h"

#include <vector>
#include <string>

class Indexer;

class SingleEndMerger: public Merger {
public:
	std::vector<unsigned> tmp;
	std::vector<unsigned> tmp1;

	SingleEndMerger(Indexer* indexer, unsigned distThresh) :
			Merger(indexer, distThresh) {
		tmp.reserve(TMPVEC_PREALLOC_SIZE);
		tmp1.reserve(TMPVEC_PREALLOC_SIZE);
	}

	virtual void findCandidatesAll(Read& read, std::vector<unsigned>& candidates) = 0;

	virtual bool findCandidatesAndMappingsK(unsigned& k, Read& read,
			std::vector<unsigned>& candidates, std::vector<ReadMapping>& results,
			SingleEndVerifier& verifier) = 0;

	virtual bool findCandidatesAndExactMappingsK(unsigned& k, Read& read,
			std::vector<unsigned>& candidates, std::vector<ReadMapping>& results,
			SingleEndVerifier& verifier) = 0;
};

class SingleEndIndelMerger: public SingleEndMerger {
public:
	SingleEndIndelMerger(Indexer* indexer, unsigned distThresh) :
			SingleEndMerger(indexer, distThresh) {
	}

	void findCandidatesAll(Read& read, std::vector<unsigned>& candidates);

	bool findCandidatesAndMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);

	bool findCandidatesAndExactMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);
};

class SingleEndHeuristicIndelMerger: public SingleEndMerger {
public:
	SingleEndHeuristicIndelMerger(Indexer* indexer, unsigned distThresh) :
			SingleEndMerger(indexer, distThresh) {
		if (distThresh >= 4) {
			bitVectorThresh = 4;
		} else {
			bitVectorThresh = distThresh;
		}
	}

	void findCandidatesAll(Read& read, std::vector<unsigned>& candidates);

	bool findCandidatesAndMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);

	bool findCandidatesAndExactMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);

private:
	unsigned bitVectorThresh;
};

class SingleEndHammingMerger: public SingleEndMerger {
public:
	SingleEndHammingMerger(Indexer* indexer, unsigned distThresh) :
			SingleEndMerger(indexer, distThresh) {
	}

	void findCandidatesAll(Read& read, std::vector<unsigned>& candidates);

	bool findCandidatesAndMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);

	bool findCandidatesAndExactMappingsK(unsigned& k, Read& read, std::vector<unsigned>& candidates,
			std::vector<ReadMapping>& results, SingleEndVerifier& verifier);
};

#endif
