// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#include "input/inputreadqueue.h"

InputReadQueue::InputReadQueue(unsigned maxReads, unsigned minOutstanding, unsigned maxOutstanding) :
			maxReads(maxReads), minOutstanding(minOutstanding), maxOutstanding(maxOutstanding),
			readsEnqueued(0), readsDequeued(0), nextReadFound(true) {
	pthread_mutex_init(&queueMutex, NULL);
	pthread_cond_init(&consumerCond, NULL);
	pthread_cond_init(&producerCond, NULL);
}

InputReadQueue::~InputReadQueue() {
	pthread_mutex_destroy(&queueMutex);
	pthread_cond_destroy(&producerCond);
	pthread_cond_destroy(&consumerCond);
}

bool InputReadQueue::getRead(ReadInfo** inputReadStruct) {
	bool ret;
	pthread_mutex_lock(&queueMutex);
	while(readQueue.size() == 0 && readsEnqueued < maxReads && nextReadFound) {
		pthread_cond_signal(&producerCond);
		pthread_cond_wait(&consumerCond, &queueMutex);
	}
	if(readQueue.size() > 0) {
		*inputReadStruct = readQueue.front();
		readQueue.pop();
		readsDequeued++;
		ret = true;
		if(readQueue.size() < minOutstanding) {
			pthread_cond_signal(&producerCond);
		}
	} else {
		ret = false;
	}
	pthread_mutex_unlock(&queueMutex);
	return ret;
}

// Meant to be executed as a thread.
void InputReadQueue::enqueueReads() {
	while(readsEnqueued < maxReads && nextReadFound) {
		pthread_mutex_lock(&queueMutex);
		while(readQueue.size() >= maxOutstanding) {
			pthread_cond_wait(&producerCond, &queueMutex);
		}
		nextReadFound = nextRead();
		if(nextReadFound) {
			pthread_cond_signal(&consumerCond);
		}
		pthread_mutex_unlock(&queueMutex);
	}
}
