// ============================================================================
//                    Hobbes Genome Sequence Mapping
// ============================================================================
// Copyright (c) 2011-2012, The Regents of the University of California, Irvine
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of The University of California nor the names of
//       its contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL KNUT REINERT OR THE FU BERLIN BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// ============================================================================

#include "globals.h"
#include "index/indexer.h"
#include "thread/indexbuildworker.h"
#include <algorithm>
#include <stdio.h>
#include <time.h>

#ifndef INDEX_BUILD_WORKER_PROGRESS_MIN
#define INDEX_BUILD_WORKER_PROGRESS_MIN 1000
#endif

#ifndef INDEX_BUILD_WORKER_PROGRESS_MAX
#define INDEX_BUILD_WORKER_PROGRESS_MAX 10000
#endif

using namespace std;

template<>
void IndexBuildWorker<BitVecTriple>::run() {
	srand(time(NULL));
	unsigned updateInterval = rand() % (INDEX_BUILD_WORKER_PROGRESS_MAX - INDEX_BUILD_WORKER_PROGRESS_MIN) + INDEX_BUILD_WORKER_PROGRESS_MIN;
	elementCount = 0;
	BitvectorType align = 0;
	if (gramGen.firstGram(str, startPos, endPos)) {
		BitvectorType bitVec = generateBitVector(align, str, gramGen.getGramPos(),
				gramGen.gramLen, endPos + 1);
		BitVecPair bitVecPair(gramGen.getGramPos(), bitVec);
		*targetPtr++ = BitVecTriple(gramGen.getGramHash(), bitVecPair);
		elementCount++;
		if (progressIndicator && (elementCount % updateInterval == 0)) {
			progressIndicator->incrementProgress(updateInterval);
		}
		while (gramGen.nextGram(str)) {
			BitvectorType bitVec = generateBitVector(align, str,
					gramGen.getGramPos(), gramGen.gramLen, endPos + 1);
			BitVecPair bitVecPair(gramGen.getGramPos(), bitVec);
			*targetPtr++ = BitVecTriple(gramGen.getGramHash(), bitVecPair);
			elementCount++;
			if (progressIndicator && (elementCount % updateInterval == 0)) {
				progressIndicator->incrementProgress(updateInterval);
			}
		}
	}
	sort(startPtr, targetPtr, BitVecTriple::cmp);
}
